﻿using System;
using Consulenza.ReportWriter.Business;
using Consulenza.ReportWriter.Business.OBJ_PDF;
using System.Data;
using Consulenza.ReportWriter.Business.CHART_PDF;
using Consulenza.ReportWriter.Business.Entity;
using System.Collections.Generic;
using Consulenza.ReportCommon;
using System.Linq;

namespace Consulenza.ReportWriter.Manager.Section.Unica
{
    /// <summary>
    /// S20.PatrimonioFideuramRappresentazionePerRating idSezione = 63
    /// </summary>
    public class S20 : Entity.Section
    {
        public S20(EnvironmentFacade environmentFacade, int idSection)
            : base(environmentFacade, idSection)
        {
            try
            {
                Draw();
            }
            catch (Exception ex)
            {
                SectionLogger.Write("S20", ex.Message, SectionLoggerMessageLevel.E, EnvironmentFacade.ReportEnvironment);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        protected override sealed void Draw()
        {
            var dati = GetDataSet();

            AddElement(new SpacePDF(20));
            if (presenzaGrafico() && dati.Tables["Patrimonio"].Rows.Count > 0)
            {
                AddElement(new FormattedTextAreaPDF(getTesto1(), EnvironmentFacade.RendererFacade.XLeftLimit) {FontSize=7, TextHorizontalAlign = ceTe.DynamicPDF.TextAlign.Justify });
                AddElement(new SpacePDF(15));

                #region Tabella titoli obbligazionari

                var tabellaTitoliObbligazionari = new TablePDF(EnvironmentFacade.RendererFacade.XLeftLimit, dati.Tables["Patrimonio"])
                   {
                       Style = Style.ConsulenzaUnica,
                       Header = false,
                       Footer = false,
                       AlternateRow = false,
                       RowHeight = 25
                   };
                tabellaTitoliObbligazionari.Columns.Add(new ColumnPDF("descrizione", 190, HorizontalAlignmentType.Sinistra, true, true, 7, ColumnType.Testo, "descrizione", string.Empty));
                tabellaTitoliObbligazionari.Columns.Add(new ColumnPDF("valore", 330, HorizontalAlignmentType.Sinistra, false, false, 7, ColumnType.Testo, "valore", string.Empty) { PaddingLeft = 5 });

                AddElement(tabellaTitoliObbligazionari);
                AddElement(new SpacePDF(25));

                #endregion

                #region Grafico a barre
                if (presenzaGrafico() && dati.Tables[1].Rows.Count > 0)
                {
                    var graficoBarre = new StackedPDF(EnvironmentFacade.RendererFacade.XLeftLimit + 110, 1F)
                    {
                        HeightSingleBar = 25,
                        Width = 400,
                        LabelFormatAxisY = isControvalore() ? FormatType.Intero : FormatType.Decimale2,
                        ShowLineAxisX = true,
                        PageBreak = true,
                        DinamicFloatXEtichettaLateraleBarra = true,
                        MinorGridAxisY = true,
                        RemoveWhiteSpaceToTheRightAxisX = true,
                        RemoveWhiteSpaceBetweenChartAndAxisX = true,
                        MarginAxisY = 0,
                        MarginAxisYFromTop = 0
                    };
                    graficoBarre.Headers.Add(new FormattedTextAreaPDF("Rating", EnvironmentFacade.RendererFacade.XLeftLimit, 100) { BackGroundColor = ColorPDF.Standard_Grigio_SfondoColonnaTabella, FontBold = true, FontSize = 7, FontColor = ColorPDF.ConsulenzaUnica_Rosso, AutoIncrementYWritable = false, BackGroundMarginLeft = 5 });
                    graficoBarre.Headers.Add(new FormattedTextAreaPDF(string.Format("Distribuzione ({0})", isControvalore() ? "€" : "%"), EnvironmentFacade.RendererFacade.XLeftLimit + 110, 410) { BackGroundColor = ColorPDF.Standard_Grigio_SfondoColonnaTabella, FontBold = true, FontSize = 7, FontColor = ColorPDF.ConsulenzaUnica_Rosso, BackGroundMarginLeft = 5 });

                    var dtGraficoBarre = dati.Tables["Rating"].AsEnumerable().OrderBy(r => r.Field<int>("Ordinamento")).CopyToDataTable();
                    var serieCollezione = new List<Serie> { new Serie { Name = "Rating", Border = true } };

                    foreach (DataRow item in dtGraficoBarre.Rows)
                    {
                        serieCollezione[0].Points.Add(new Point
                        {
                            ShowLabelAxisY = true,
                            LabelAxisY = isControvalore() ? Helper.FormatCurrency(item["Controvalore"].ToString()) : Helper.FormatDecimal(item["Percentuale"].ToString(), 2),
                            FontSizeLabelAxisY = 7,

                            Color = new ColorPDF(Convert.ToInt32(item["Red"]), Convert.ToInt32(item["Green"]), Convert.ToInt32(item["Blue"])),
                            Value = isControvalore() ? (Convert.ToDouble(item["Controvalore"]) == 0 ? 0.01 : Convert.ToDouble(item["Controvalore"])) : Convert.ToDouble(item["Percentuale"]) == 0 ? 0.01 : Convert.ToDouble(item["Percentuale"]), // Convert.ToDouble(Helper.FormatPercentage(Convert.ToDecimal(item["Percentuale"]), 2)),

                            ShowLabelAxisX = true,
                            LabelAxisX = item["Rating"].ToString(),
                            FontSizeLabelAxisX = 7,
                            FontBoldLabelAxisY = false
                        });
                    }

                    graficoBarre.SeriesCollection = serieCollezione;
                    graficoBarre.MinorGridAxisY = true;
                    AddElement(graficoBarre);
                    AddElement(new SpacePDF(20));

                #endregion
                }
            }

            if (presenzaTabella() && dati.Tables["ProdottoPerRating"].Rows.Count > 0)
            {
                #region Testo introduttivo tabella prodotti

                AddElement(new SpacePDF(15));
                //var testoTabellaProdotti = Helper.ReplaceVariables(_testi.Rows[0]["testo3"].ToString(), EnvironmentFacade.ReportEnvironment);
                if (dati.Tables["ProdottoPerRating"].Rows.Count > 0)
                    AddElement(new FormattedTextAreaPDF(getTesto2(), EnvironmentFacade.RendererFacade.XLeftLimit) {FontSize=7, TextHorizontalAlign = ceTe.DynamicPDF.TextAlign.Justify });
                AddElement(new SpacePDF(15));

                #endregion

                #region Tabella Prodotti X Rating

                var tabellaProdottiPerRating = new TablePDF(EnvironmentFacade.RendererFacade.XLeftLimit, dati.Tables["ProdottoPerRating"], dati.Tables["ProdottoPerRatingFooter"])
                  {
                      Style = Style.ConsulenzaUnica,
                      Header = true,
                      Footer = true,
                      AlternateRow = false,
                      HeaderHeight = 30,
                      HeaderMargin = 2
                  };

                #region Columns e FooterColumns

                tabellaProdottiPerRating.Columns.Add(new ColumnPDF("Immagine", 165, HorizontalAlignmentType.Sinistra, false, false, 7, ColumnType.Objectpdf, string.Empty, "Obbligazione") { HeaderFontSize = 7, FontSize = 7, DeltaYContent = 4, PaddingLeft = 2 });
                //tabellaProdottiPerRating.Columns.Add(new ColumnPDF("Prodotto", 165, HorizontalAlignmentType.Sinistra, false, false, 7, ColumnType.Testo, "Prodotto", "Obbligazione"));
                tabellaProdottiPerRating.Columns.Add(new ColumnPDF("Emittente", 150, HorizontalAlignmentType.Sinistra, false, false, 7, ColumnType.Testo, "Emittente", "Emittente") { HeaderFontSize = 7, FontSize = 7});
                tabellaProdottiPerRating.Columns.Add(new ColumnPDF("Rating", 65, HorizontalAlignmentType.Sinistra, false, false, 7, ColumnType.Testo, "Rating", "Rating") { HeaderFontSize = 7, FontSize = 7, HeaderPaddingLeft = 3, PaddingLeft = 3 });
                tabellaProdottiPerRating.Columns.Add(new ColumnPDF("Controvalore", 100, HorizontalAlignmentType.Sinistra, false, true, 7, ColumnType.Decimale, "Controvalore", "Controvalore attuale (€)") { HeaderFontSize = 7, FontSize = 7, PaddingRight = 4 });
                tabellaProdottiPerRating.Columns.Add(new ColumnPDF("Peso", 40, HorizontalAlignmentType.Sinistra, false, true, 7, ColumnType.Decimale, "Peso", "Peso (%)") { HeaderFontSize = 7, FontSize = 7, PaddingRight = 4 });
                tabellaProdottiPerRating.FooterColumns.Add(new ColumnPDF("Obbligazione", 380, HorizontalAlignmentType.Sinistra, true, true, 7, ColumnType.Testo));
                tabellaProdottiPerRating.FooterColumns.Add(new ColumnPDF("Controvalore", 100, HorizontalAlignmentType.Destra, true, true, 7, ColumnType.Decimale) { PaddingRight = 4 });
                tabellaProdottiPerRating.FooterColumns.Add(new ColumnPDF("Peso", 40, HorizontalAlignmentType.Destra, true, true, 7, ColumnType.Decimale) { HeaderPaddingLeft = 3});

                int a = 0;
                foreach (var item in dati.Tables[3].Rows)
                {
                    tabellaProdottiPerRating.Cells[4, a].HorizontalAlignment = HorizontalAlignmentType.Destra;
                    tabellaProdottiPerRating.Cells[3, a].HorizontalAlignment = HorizontalAlignmentType.Destra;
                    a++;
                }

                #endregion

                #region Cells

                int i = 0;
                foreach (DataRow row in dati.Tables["ProdottoPerRating"].Rows)
                {
                    if (Convert.ToBoolean(row["IsTotal"]))
                    {
                        // Sfondo grigio per la riga identificata con IsTotal [quella di intestazione di Rating]
                        for (int c = 0; c < tabellaProdottiPerRating.Columns.Count; c++)
                        {
                            tabellaProdottiPerRating.Cells[c, i].BackgroundColor = ColorPDF.Standard_Grigio_SfondoColonnaTabella;
                        }
                        var listaOggetti = new List<ObjectPDF>                    
                            {
                                new RectanglePDF(10, 10, new ColorPDF(row["CodiceHexRating"].ToString())),   
                                new FormattedTextAreaPDF
                                {
                                    Text = row["Prodotto"].ToString(),
                                    FontBold = true,
                                    FontColor = ColorPDF.Nero,
                                    DeltaX = 12F,
                                    FontSize=7,
                                }
                            };
                        tabellaProdottiPerRating.Cells[0, i].ValueObjectList.AddRange(listaOggetti);
                    }
                    else
                    {
                        var listaOggetti = new List<ObjectPDF>                    
                            {
                                new RectanglePDF(10, 10, ColorPDF.GetBy_AreaBisogno(row["CodiceAreaBisogno"].ToString())),   
                                new FormattedTextAreaPDF
                                {
                                    Text = new AreaBisogno(row["CodiceAreaBisogno"].ToString()).Abbreviazione,
                                    FontBold = true,
                                    FontColor = ColorPDF.Bianco,
                                    DeltaX = 2.5F,
                                    FontSize=7,
                                },
                                new FormattedTextAreaPDF
                                {
                                    Text = row["Prodotto"].ToString(),
                                    FontBold = false,
                                    FontColor = ColorPDF.Nero,
                                    DeltaX = 12F,
                                    FontSize=7,
                                } 
                            };
                        tabellaProdottiPerRating.Cells[0, i].ValueObjectList.AddRange(listaOggetti);
                    }

                    i++;
                }

                #endregion

                AddElement(tabellaProdottiPerRating);

                #endregion
            }
        }

        /// <summary>
        /// Recupera i dati necessari alla Section restituendo un DataTable.
        /// </summary>
        /// <returns></returns>
        protected sealed override DataTable GetDataTable()
        {
            return null;
        }

        /// <summary>
        /// Recupera i dati necessari alla Section restituendo un DataSet.
        /// </summary>
        /// <returns></returns>
        protected sealed override DataSet GetDataSet()
        {
            var ds = new DataSet();
            ds.Tables.Add(new DataTable("Patrimonio"));
            ds.Tables.Add(new DataTable("Rating"));
            ds.Tables.Add(new DataTable("Prodotto"));
            ds.Tables.Add(new DataTable("ProdottoPerRating"));
            ds.Tables.Add(new DataTable("ProdottoPerRatingFooter"));
            //var chiaveClientePb = EnvironmentFacade.ReportEnvironment.Cliente.Chiave;
           
            const string stringReplaceIfZero = " ";

            //var patrimonioCasa = new DataServices.PatrimonioCasa(chiaveClientePb);
            //var dataDettaglioProdotti = patrimonioCasa.GetListaProdotti();
            //var dataRating = patrimonioCasa.GetDistribuzioneRating();
            //var displayInfo = new DataServices.DisplayInfo(chiaveClientePb).GetDisplayInfo().ratingDisplayInfos.ToList();

            var dataDettaglioProdotti = datiSeiUnico.patrimonioUnit().patrimonioCasa.dettaglioProdotti.dettaglioProdotti;
            var dataRating = datiSeiUnico.patrimonioUnit().patrimonioCasa.distribuzioneRating;
            var displayInfo = datiSeiUnico.displayInfos().ratingDisplayInfos.ToList();   

            #region Patrimonio titoli obbligazionari

            //var dtPatrimonio = new DataTable();
            ds.Tables["Patrimonio"].Columns.Add(new DataColumn("Descrizione", typeof(string)));
            ds.Tables["Patrimonio"].Columns.Add(new DataColumn("Valore", typeof(string)));
            if (dataRating.distribuzione.totaleRapprNoCC > 0)
                ds.Tables["Patrimonio"].Rows.Add("Totale titoli obbligazionari (€)", Helper.FormatCurrency(dataRating.distribuzione.totaleRapprNoCC.ToString()));

            #endregion

            #region Rating

            //var dtRating = new DataTable();
            ds.Tables["Rating"].Columns.Add(new DataColumn("Rating", typeof(string)));
            ds.Tables["Rating"].Columns.Add(new DataColumn("Red", typeof(int)));
            ds.Tables["Rating"].Columns.Add(new DataColumn("Green", typeof(int)));
            ds.Tables["Rating"].Columns.Add(new DataColumn("Blue", typeof(int)));
            ds.Tables["Rating"].Columns.Add(new DataColumn("Controvalore", typeof(decimal)));
            ds.Tables["Rating"].Columns.Add(new DataColumn("Percentuale", typeof(decimal)));
            ds.Tables["Rating"].Columns.Add(new DataColumn("Ordinamento", typeof(int)));

            foreach (var itemDistribuzione in dataRating.distribuzione.elencoSlice)
            {
                if (itemDistribuzione.sliceRappresentabile)
                {
                    //var mapDisplayInfo = DataServices.DisplayInfo.TryGetValue(displayInfo, itemDistribuzione.codice);
                    //var colore = new ColorPDF(mapDisplayInfo.fill);
                    //var ordinamento = mapDisplayInfo.order;
                    var mapDisplayInfo = displayInfo.Where(l => l.key == itemDistribuzione.codice).ToArray();
                    var colore = new ColorPDF(mapDisplayInfo[0].value.fill);
                    var ordinamento = mapDisplayInfo[0].value.order;
                    ds.Tables["Rating"].Rows.Add(itemDistribuzione.descrizione, colore.Red, colore.Green, colore.Blue, itemDistribuzione.ctv, itemDistribuzione.peso * 100, ordinamento);
                }
            }
            if (ds.Tables["Rating"].Rows.Count > 0)
                ds.Tables["Rating"].AsEnumerable().OrderBy(r => r.Field<int>("Ordinamento")).CopyToDataTable();

            #endregion

            #region Prodotti

            //var dtProdotti = new DataTable();
            ds.Tables["Prodotto"].Columns.Add(new DataColumn("CodiceAreaBisogno", typeof(string)));
            ds.Tables["Prodotto"].Columns.Add(new DataColumn("OrdinamentoRating", typeof(int)));
            ds.Tables["Prodotto"].Columns.Add(new DataColumn("CodiceHexRating", typeof(string)));

            ds.Tables["Prodotto"].Columns.Add(new DataColumn("Prodotto", typeof(string)));
            ds.Tables["Prodotto"].Columns.Add(new DataColumn("Emittente", typeof(string)));
            ds.Tables["Prodotto"].Columns.Add(new DataColumn("FasciaRating", typeof(string)));
            ds.Tables["Prodotto"].Columns.Add(new DataColumn("Rating", typeof(string)));
            ds.Tables["Prodotto"].Columns.Add(new DataColumn("Controvalore", typeof(decimal)));
            ds.Tables["Prodotto"].Columns.Add(new DataColumn("Peso", typeof(string)));

            ds.Tables["Prodotto"].Columns.Add(new DataColumn("FontBold", typeof(bool)));
            ds.Tables["Prodotto"].Columns.Add(new DataColumn("ReplaceIfZero", typeof(string)));
            ds.Tables["Prodotto"].Columns.Add(new DataColumn("CanLastRow", typeof(bool)));
            ds.Tables["Prodotto"].Columns.Add(new DataColumn("IsTotal", typeof(bool)));
  
            foreach (var itemRating in dataRating.distribuzionePerProdotti.listaProdotti)
            {
                if (itemRating.rappresentabile)
                {
                    var itemPosizione = dataDettaglioProdotti.FirstOrDefault(o => o.chiavePosizionePortafoglio.Equals(itemRating.chiaveAggregazione));

                    //var mapDisplayInfo = DataServices.DisplayInfo.TryGetValue(displayInfo, dataRating.distribuzione.elencoSlice.FirstOrDefault(o => o.descrizione.Equals(itemRating.listaSlice[0].descrizione)).codice);
                    var mapDisplayInfo = displayInfo.Where(l => l.key == dataRating.distribuzione.elencoSlice.FirstOrDefault(o => o.descrizione.Equals(itemRating.listaSlice[0].descrizione)).codice).ToArray();
                    
                    ds.Tables["Prodotto"].Rows.Add(
                        itemPosizione.area, //CodiceAreaBisogno 
                        mapDisplayInfo[0].value.order, //OrdinamentoRating
                        mapDisplayInfo[0].value.fill, //CodiceHexRating

                        itemPosizione.prodotto.nomeProdotto,//Prodotto
                        itemPosizione.prodotto.emittente, //Emittente
                        itemRating.listaSlice[0].descrizione, //FasciaRating
                        itemPosizione.prodotto.tipoRating, //Rating
                        itemRating.listaSlice[0].ctv, //Controvalore
                        itemRating.listaSlice[0].peso * 100, //Peso

                        false, //FontBold
                        " ", //ReplaceIfZero
                        true, //CanLastRow
                        false //IsTotal

                        );
                }
            }
            if (ds.Tables["Prodotto"].Rows.Count > 0)
                ds.Tables["Prodotto"].AsEnumerable().OrderBy(r => r.Field<int>("OrdinamentoRating"));
            #endregion

            #region Prodotti raggruppati per Rating

            //var dtProdottiPerRating = dtProdotti.Clone(); // Clono la struttura
            ds.Tables["ProdottoPerRating"].Columns.Add(new DataColumn("CodiceAreaBisogno", typeof(string)));
            ds.Tables["ProdottoPerRating"].Columns.Add(new DataColumn("OrdinamentoRating", typeof(int)));
            ds.Tables["ProdottoPerRating"].Columns.Add(new DataColumn("CodiceHexRating", typeof(string)));

            ds.Tables["ProdottoPerRating"].Columns.Add(new DataColumn("Prodotto", typeof(string)));
            ds.Tables["ProdottoPerRating"].Columns.Add(new DataColumn("Emittente", typeof(string)));
            ds.Tables["ProdottoPerRating"].Columns.Add(new DataColumn("FasciaRating", typeof(string)));
            ds.Tables["ProdottoPerRating"].Columns.Add(new DataColumn("Rating", typeof(string)));
            ds.Tables["ProdottoPerRating"].Columns.Add(new DataColumn("Controvalore", typeof(decimal)));
            ds.Tables["ProdottoPerRating"].Columns.Add(new DataColumn("Peso", typeof(string)));

            ds.Tables["ProdottoPerRating"].Columns.Add(new DataColumn("FontBold", typeof(bool)));
            ds.Tables["ProdottoPerRating"].Columns.Add(new DataColumn("ReplaceIfZero", typeof(string)));
            ds.Tables["ProdottoPerRating"].Columns.Add(new DataColumn("CanLastRow", typeof(bool)));
            ds.Tables["ProdottoPerRating"].Columns.Add(new DataColumn("IsTotal", typeof(bool)));

            var viewRating = new DataView(ds.Tables["Prodotto"]) { Sort = "OrdinamentoRating" };
            var dtRatingDistinct = viewRating.ToTable(true, "FasciaRating", "OrdinamentoRating", "CodiceHexRating");

            var fasciaRating = string.Empty;

            #region Prodotti delle Risorse Finanziarie

            foreach (DataRow rowRating in dtRatingDistinct.Rows)
            {
                fasciaRating = rowRating["FasciaRating"].ToString();

                // subset di prodotti filtrati per Rating
                var dt = ds.Tables["Prodotto"].Select("FasciaRating='" + fasciaRating + "'").OrderByDescending(r => r.Field<Decimal>("Controvalore")).CopyToDataTable();

                #region riga con background grigio di intestazione per ogni area

                DataRow rowProdottiPerRating = ds.Tables["ProdottoPerRating"].NewRow();

                rowProdottiPerRating["CodiceHexRating"] = rowRating["CodiceHexRating"];

                rowProdottiPerRating["Prodotto"] = rowRating["FasciaRating"];
                rowProdottiPerRating["Controvalore"] = dataRating.distribuzione.elencoSlice.FirstOrDefault(o => o.descrizione.Equals(rowRating["FasciaRating"].ToString())).ctv;// dt.AsEnumerable().Sum(x => x.Field<decimal>("Controvalore"));
                rowProdottiPerRating["Peso"] = dataRating.distribuzione.elencoSlice.FirstOrDefault(o => o.descrizione.Equals(rowRating["FasciaRating"].ToString())).peso * 100;//dt.AsEnumerable().Sum(x => x.Field<decimal>("Peso"));

                rowProdottiPerRating["FontBold"] = true;
                rowProdottiPerRating["ReplaceIfZero"] = stringReplaceIfZero;
                rowProdottiPerRating["CanLastRow"] = false;
                rowProdottiPerRating["IsTotal"] = true;

                ds.Tables["ProdottoPerRating"].Rows.Add(rowProdottiPerRating);

                #endregion

                // I prodotti della dtProdotti vengono mergiati sulla dtProdottiPerArea
                ds.Tables["ProdottoPerRating"].Merge(dt);

            }
            ds.Tables["ProdottoPerRating"].AsEnumerable().OrderBy(r => r.Field<int>("OrdinamentoRating"));
            #endregion

            #region Footer 
            ds.Tables["ProdottoPerRatingFooter"].Columns.Add(new DataColumn("Obbligazione", typeof(string)));
            ds.Tables["ProdottoPerRatingFooter"].Columns.Add(new DataColumn("Controvalore", typeof(string)));
            ds.Tables["ProdottoPerRatingFooter"].Columns.Add(new DataColumn("Peso", typeof(string)));

            ds.Tables["ProdottoPerRatingFooter"].Rows.Add(
                                                        "Totale titoli obbligazionari",
                                                        Helper.FormatCurrency(dataRating.distribuzione.totaleRapprNoCC.ToString()),
                                                        (dataRating.distribuzione.pesoTot*100).ToString()
                                                    );

            #endregion

            #endregion

            //ds.Tables.Add(dtPatrimonio);
            //ds.Tables.Add(dtRating.AsEnumerable().OrderBy(r => r.Field<int>("Ordinamento")).CopyToDataTable());
            //ds.Tables.Add(dtProdotti);
            //ds.Tables.Add(dtProdottiPerRating);

            return ds;
        }

        public virtual string getTesto1()
        {
            var testo = GetText();
            if (!isControvalore())
                return Helper.ReplaceVariables(testo.Rows[0]["testo1"].ToString(), EnvironmentFacade.ReportEnvironment);
            else
                return Helper.ReplaceVariables(testo.Rows[0]["testo2"].ToString(), EnvironmentFacade.ReportEnvironment);
        }

        public virtual string getTesto2()
        {
             var testo = GetText();
             return Helper.ReplaceVariables(testo.Rows[0]["testo3"].ToString(), EnvironmentFacade.ReportEnvironment);
        }

        public virtual bool presenzaGrafico()
        {
            return GetOption<Opzione4_1>().Valore;
        }

        public virtual bool isControvalore()
        {
            return !GetOption<Opzione3>().Valore;
        }

        public virtual bool presenzaTabella()
        {
            return GetOption<Opzione4_2>().Valore;
        }
    }
}
