﻿using System;
using Consulenza.ReportWriter.Business;
using Consulenza.ReportWriter.Business.OBJ_PDF;
using System.Data;
using Consulenza.ReportWriter.Business.CHART_PDF;
using Consulenza.ReportWriter.Business.Entity;
using System.Collections.Generic;
using Consulenza.ReportCommon;
using System.Linq;

namespace Consulenza.ReportWriter.Manager.Section.Unica
{
    /// <summary>
    /// S22.PatrimonioFideuramRappresentazionePerScadenze idSezione =65
    /// </summary>
    public class S22 : Entity.Section
    {
       
        public S22(EnvironmentFacade environmentFacade, int idSection)
            : base(environmentFacade, idSection)
        {
            try
            {
               
                Draw();
            }
            catch (Exception ex)
            {
                SectionLogger.Write("S22", ex.Message, SectionLoggerMessageLevel.E, EnvironmentFacade.ReportEnvironment);
            }
        }
 
        protected override sealed void Draw()
        {
            var dati = GetDataSet();
      
            AddElement(new SpacePDF(20));
            if (presenzaGraficoBarre() && dati.Tables[1].Rows.Count > 0)
            {
                #region Testo introduttivo grafico
                AddElement(new FormattedTextAreaPDF(getTesto1(), EnvironmentFacade.RendererFacade.XLeftLimit) { FontSize = 7, TextHorizontalAlign = ceTe.DynamicPDF.TextAlign.Justify });
                AddElement(new SpacePDF(15));

                #endregion

                #region Tabella titoli obbligazionari

                var tabellaTitoliObbligazionari = new TablePDF(EnvironmentFacade.RendererFacade.XLeftLimit, dati.Tables[0])
                {
                    Style = Style.ConsulenzaUnica,
                    Header = false,
                    Footer = false,
                    AlternateRow = false,
                    RowHeight = 25
                };
                tabellaTitoliObbligazionari.Columns.Add(new ColumnPDF("descrizione", 190, HorizontalAlignmentType.Sinistra, true, true, 7, ColumnType.Testo, "descrizione", string.Empty));
                tabellaTitoliObbligazionari.Columns.Add(new ColumnPDF("valore", 330, HorizontalAlignmentType.Sinistra, false, false, 7, ColumnType.Testo, "valore", string.Empty) { PaddingLeft = 5 });

                AddElement(tabellaTitoliObbligazionari);
                AddElement(new SpacePDF(25));

                #endregion

                if (presenzaGraficoBarre() && dati.Tables[1].Rows.Count > 0)
                {
                    #region Grafico a barre

                    var graficoBarre = new StackedPDF(EnvironmentFacade.RendererFacade.XLeftLimit + 110, 1F)
                    {
                        HeightSingleBar = 25,
                        Width = 400,
                        LabelFormatAxisY = isControvalore() ? FormatType.Intero : FormatType.Decimale2,
                        ShowLineAxisX = true,
                        PageBreak = true,
                        DinamicFloatXEtichettaLateraleBarra = true,
                        MinorGridAxisY = true,
                        RemoveWhiteSpaceToTheRightAxisX = true,
                        RemoveWhiteSpaceBetweenChartAndAxisX = true,
                        MarginAxisY = 0,
                        MarginAxisYFromTop = 0
                    };
                    graficoBarre.Headers.Add(new FormattedTextAreaPDF("Scadenza", EnvironmentFacade.RendererFacade.XLeftLimit, 100) { BackGroundColor = ColorPDF.Standard_Grigio_SfondoColonnaTabella, FontBold = true, FontSize = 7, FontColor = ColorPDF.ConsulenzaUnica_Rosso, AutoIncrementYWritable = false, BackGroundMarginLeft = 5 });
                    graficoBarre.Headers.Add(new FormattedTextAreaPDF(string.Format("Distribuzione ({0})", isControvalore() ? "€" : "%"), EnvironmentFacade.RendererFacade.XLeftLimit + 110, 410) { BackGroundColor = ColorPDF.Standard_Grigio_SfondoColonnaTabella, FontBold = true, FontSize = 7, FontColor = ColorPDF.ConsulenzaUnica_Rosso, BackGroundMarginLeft = 5 });

                    var dtGraficoBarre = dati.Tables[1];
                    var serieCollezione = new List<Serie> { new Serie { Name = "Scadenza", Border = true } };

                    foreach (DataRow item in dtGraficoBarre.Rows)
                    {
                        serieCollezione[0].Points.Add(new Point
                        {
                            ShowLabelAxisY = true,
                            LabelAxisY = isControvalore() ? Helper.FormatCurrency(item["Controvalore"].ToString()) : Helper.FormatDecimal(item["Percentuale"].ToString(), 2),
                            FontSizeLabelAxisY = 7,

                            Color = new ColorPDF(Convert.ToInt32(item["Red"]), Convert.ToInt32(item["Green"]), Convert.ToInt32(item["Blue"])),
                            Value = isControvalore() ? (Convert.ToDouble(item["Controvalore"]) == 0 ? 0.01 : Convert.ToDouble(item["Controvalore"])) : Convert.ToDouble(item["Percentuale"]) == 0 ? 0.01 : Convert.ToDouble(item["Percentuale"]), // Convert.ToDouble(Helper.FormatPercentage(Convert.ToDecimal(item["Percentuale"]), 2)),

                            ShowLabelAxisX = true,
                            LabelAxisX = item["Scadenza"].ToString(),
                            FontSizeLabelAxisX = 7,
                            FontBoldLabelAxisY = false
                        });
                    }

                    graficoBarre.SeriesCollection = serieCollezione;
                    if (isControvalore())
                        graficoBarre.RemoveWhiteSpaceToTheRightAxisX = true;
             //       graficoBarre.MaximumValueAxisY = 100;

                    AddElement(graficoBarre);
                    AddElement(new SpacePDF(20));

                    #endregion
                }

                #region Nota del Patrimonio titoli obbligazionari
                if (getNote1().Length > 0)
                {
                    AddElement(new FormattedTextAreaPDF(getNote1(), EnvironmentFacade.RendererFacade.XLeftLimit) { FontSize = 7, TextHorizontalAlign = ceTe.DynamicPDF.TextAlign.Justify });
                    AddElement(new SpacePDF(20));
                }
                #endregion
            }

            

            if (presenzaTabella() && dati.Tables[3].Rows.Count > 0)
            {
                #region Testo introduttivo tabella prodotti

                AddElement(new SpacePDF(15));
                

                AddElement(new FormattedTextAreaPDF(getTesto2(), EnvironmentFacade.RendererFacade.XLeftLimit) {FontSize = 7, TextHorizontalAlign = ceTe.DynamicPDF.TextAlign.Justify });
                AddElement(new SpacePDF(15));

                #endregion

                #region Tabella Prodotti X Scadenza

                var tabellaProdottiPerScadenza = new TablePDF(EnvironmentFacade.RendererFacade.XLeftLimit, dati.Tables[3], dati.Tables[4])
                {
                    Style = Style.ConsulenzaUnica,
                    Header = true,
                    Footer = true,
                    AlternateRow = false,
                    HeaderHeight = 30,
                    HeaderMargin = 2
                };

                #region Columns e FooterColumns

                tabellaProdottiPerScadenza.Columns.Add(new ColumnPDF("Immagine", 15 + 150, HorizontalAlignmentType.Sinistra, false, false, 7, ColumnType.Objectpdf, string.Empty, "Obbligazione") { HeaderFontSize = 7, FontSize = 7, DeltaYContent = 4, PaddingLeft = 2 });
                //tabellaProdottiPerScadenza.Columns.Add(new ColumnPDF("Prodotto", 150, HorizontalAlignmentType.Sinistra, false, false, 6, ColumnType.Testo, "Prodotto", "Obbligazione") { HeaderFontSize = 7, HeaderFontBold = true});
                tabellaProdottiPerScadenza.Columns.Add(new ColumnPDF("Emittente", 150, HorizontalAlignmentType.Sinistra, false, false, 7, ColumnType.Testo, "Emittente", "Emittente") { HeaderFontSize = 7, FontSize = 7 });
                tabellaProdottiPerScadenza.Columns.Add(new ColumnPDF("Rating", 55, HorizontalAlignmentType.Sinistra, false, false, 7, ColumnType.Testo, "Scadenza", "Scadenza") { HeaderFontSize = 7, FontSize = 7, HeaderPaddingLeft = 3, PaddingLeft = 3 });
                tabellaProdottiPerScadenza.Columns.Add(new ColumnPDF("Controvalore", 100, HorizontalAlignmentType.Destra, false, true, 7, ColumnType.Decimale, "Controvalore", "Controvalore attuale (€)") { HeaderFontSize = 7, FontSize = 7, PaddingRight = 4 });
                tabellaProdottiPerScadenza.Columns.Add(new ColumnPDF("Peso", 50, HorizontalAlignmentType.Destra, false, true, 7, ColumnType.Decimale, "Peso", "Peso (%)") { HeaderFontSize = 7, FontSize = 7, PaddingRight = 4 });

                tabellaProdottiPerScadenza.FooterColumns.Add(new ColumnPDF("Prodotto", 380, HorizontalAlignmentType.Sinistra, true, true, 7, ColumnType.Testo));
                tabellaProdottiPerScadenza.FooterColumns.Add(new ColumnPDF("Controvalore", 90, HorizontalAlignmentType.Destra, true, true, 7, ColumnType.Decimale) { PaddingRight = 4 });
                tabellaProdottiPerScadenza.FooterColumns.Add(new ColumnPDF("Peso", 50, HorizontalAlignmentType.Destra, true, true, 7, ColumnType.Decimale) { HeaderPaddingLeft = 3});

                int a = 0;
                foreach (var item in dati.Tables[3].Rows)
                {
                    tabellaProdottiPerScadenza.Cells[4, a].HorizontalAlignment = HorizontalAlignmentType.Destra;
                    tabellaProdottiPerScadenza.Cells[3, a].HorizontalAlignment = HorizontalAlignmentType.Destra;
                    a++;
                }
                #endregion

                #region Cells

                int i = 0;
                foreach (DataRow row in dati.Tables[3].Rows)
                {
                    if (Convert.ToBoolean(row["IsTotal"]))
                    {
                        // Sfondo grigio per la riga identificata con IsTotal [quella di intestazione di Scadenza]
                        for (int c = 0; c < tabellaProdottiPerScadenza.Columns.Count; c++)
                        {
                            tabellaProdottiPerScadenza.Cells[c, i].BackgroundColor = ColorPDF.Standard_Grigio_SfondoColonnaTabella;
                        }
                        var listaOggetti = new List<ObjectPDF>                    
                            {
                                new RectanglePDF(10, 10, new ColorPDF(row["CodiceHexScadenza"].ToString())),   
                                new FormattedTextAreaPDF
                                {
                                    Text = row["Prodotto"].ToString(),
                                    FontBold = true,
                                    FontColor = ColorPDF.Nero,
                                    DeltaX = 12F,
                                    FontSize=7,
                                }
                            };
                        tabellaProdottiPerScadenza.Cells[0, i].ValueObjectList.AddRange(listaOggetti);
                    }
                    else
                    {
                        var listaOggetti = new List<ObjectPDF>                    
                            {
                                new RectanglePDF(10, 10, ColorPDF.GetBy_AreaBisogno(row["CodiceAreaBisogno"].ToString())),   
                                new FormattedTextAreaPDF
                                {
                                    Text = new AreaBisogno(row["CodiceAreaBisogno"].ToString()).Abbreviazione,
                                    FontBold = true,
                                    FontColor = ColorPDF.Bianco,
                                    DeltaX = 2.5F,
                                    FontSize=7,
                                },
                                new FormattedTextAreaPDF
                                {
                                    Text = row["Prodotto"].ToString(),
                                    FontBold = false,
                                    FontColor = ColorPDF.Nero,
                                    DeltaX = 12F,
                                    FontSize=7,
                                } 
                            };
                        tabellaProdottiPerScadenza.Cells[0, i].ValueObjectList.AddRange(listaOggetti);
                    }

                    i++;
                }

                #endregion

                AddElement(tabellaProdottiPerScadenza);

                #endregion
            }
        }

         
        protected sealed override DataTable GetDataTable()
        {
            return null;
        }

        
        protected override DataSet GetDataSet()
        {
            var ds = new DataSet();
          

            
            var bNote = getNote1().Length > 0;
           
            const string stringReplaceIfZero = " ";


            var dataDettaglioProdotti = datiSeiUnico.patrimonioUnit().patrimonioCasa.dettaglioProdotti.dettaglioProdotti.ToList();
            var dataScadenze = datiSeiUnico.patrimonioUnit().patrimonioCasa.distribuzioneScadenze;
            var displayInfo = datiSeiUnico.displayInfos().scadenzeDisplayInfos.ToList();

            #region Patrimonio titoli obbligazionari

            var dtPatrimonio = new DataTable();
            dtPatrimonio.Columns.Add(new DataColumn("Descrizione", typeof(string)));
            dtPatrimonio.Columns.Add(new DataColumn("Valore", typeof(string)));

            var testoPatrimonio = string.Format("Totale titoli obbligazionari (€){0}", bNote ? "*" : string.Empty);
            dtPatrimonio.Rows.Add(testoPatrimonio, Helper.FormatCurrency(dataScadenze.distribuzione.totaleRapprNoCC.ToString()));

            #endregion

            #region Scadenze

            var dtScadenze = new DataTable();
            dtScadenze.Columns.Add(new DataColumn("Scadenza", typeof(string)));
            dtScadenze.Columns.Add(new DataColumn("Red", typeof(int)));
            dtScadenze.Columns.Add(new DataColumn("Green", typeof(int)));
            dtScadenze.Columns.Add(new DataColumn("Blue", typeof(int)));
            dtScadenze.Columns.Add(new DataColumn("Controvalore", typeof(decimal)));
            dtScadenze.Columns.Add(new DataColumn("Percentuale", typeof(decimal)));
            dtScadenze.Columns.Add(new DataColumn("Ordinamento", typeof(int)));

            foreach (var itemDistribuzione in dataScadenze.distribuzione.elencoSlice)
            {
                if (itemDistribuzione.sliceRappresentabile)
                {
                    var mapDisplayInfo = displayInfo.FirstOrDefault(o => o.key.Equals(itemDistribuzione.codice)).value;
                    
                    var colore = new ColorPDF(mapDisplayInfo.fill);
                    var ordinamento = mapDisplayInfo.order;

                    dtScadenze.Rows.Add(itemDistribuzione.descrizione, colore.Red, colore.Green, colore.Blue, itemDistribuzione.ctv, itemDistribuzione.peso * 100, ordinamento);
                }
            }

            #endregion

            #region Prodotti

            var dtProdotti = new DataTable();
            dtProdotti.Columns.Add(new DataColumn("CodiceAreaBisogno", typeof(string)));
            dtProdotti.Columns.Add(new DataColumn("OrdinamentoScadenza", typeof(int)));
            dtProdotti.Columns.Add(new DataColumn("CodiceHexScadenza", typeof(string)));

            dtProdotti.Columns.Add(new DataColumn("Prodotto", typeof(string)));
            dtProdotti.Columns.Add(new DataColumn("Emittente", typeof(string)));
            dtProdotti.Columns.Add(new DataColumn("FasciaScadenza", typeof(string)));
            dtProdotti.Columns.Add(new DataColumn("Scadenza", typeof(string)));
            dtProdotti.Columns.Add(new DataColumn("Controvalore", typeof(decimal)));
            dtProdotti.Columns.Add(new DataColumn("Peso", typeof(decimal)));

            dtProdotti.Columns.Add(new DataColumn("FontBold", typeof(bool)));
            dtProdotti.Columns.Add(new DataColumn("ReplaceIfZero", typeof(string)));
            dtProdotti.Columns.Add(new DataColumn("CanLastRow", typeof(bool)));
            dtProdotti.Columns.Add(new DataColumn("IsTotal", typeof(bool)));

            foreach (var itemScadenza in dataScadenze.distribuzionePerProdotti.listaProdotti)
            {
                if (itemScadenza.rappresentabile)
                {
                    var itemPosizione = dataDettaglioProdotti.FirstOrDefault(o => o.chiavePosizionePortafoglio.Equals(itemScadenza.chiaveAggregazione));

                    var mapDisplayInfo = displayInfo.FirstOrDefault(o => o.key.Equals(dataScadenze.distribuzione.elencoSlice.FirstOrDefault(oo => oo.descrizione.Equals(itemScadenza.listaSlice[0].descrizione)).codice)).value;
                    

                    dtProdotti.Rows.Add(
                        itemPosizione.area, //CodiceAreaBisogno 
                        mapDisplayInfo.order, //OrdinamentoScadenza
                        mapDisplayInfo.fill, //CodiceHexScadenza

                        itemPosizione.prodotto.nomeProdotto,//Prodotto
                        itemPosizione.prodotto.emittente, //Emittente
                        itemScadenza.listaSlice[0].descrizione, //FasciaScadenza
                        itemPosizione.prodotto.dtScadenza.ToShortDateString(),
                        itemScadenza.listaSlice[0].ctv, //Controvalore
                        itemScadenza.listaSlice[0].peso * 100, //Peso

                        false, //FontBold
                        " ", //ReplaceIfZero
                        true, //CanLastRow
                        false //IsTotal
                        );
                }
            }

            #endregion

            #region Prodotti raggruppati per Scadenza

            var dtProdottiPerScadenza = dtProdotti.Clone(); // Clono la struttura

            var viewScadenze = new DataView(dtProdotti) { Sort = "OrdinamentoScadenza" };
            var dtScadenzeDistinct = viewScadenze.ToTable(true, "FasciaScadenza", "OrdinamentoScadenza", "CodiceHexScadenza");

            var fasciaScadenza = string.Empty;

            #region Prodotti delle Risorse Finanziarie

            foreach (DataRow rowScadenza in dtScadenzeDistinct.Rows)
            {
                fasciaScadenza = rowScadenza["FasciaScadenza"].ToString();

                // subset di prodotti filtrati per Area
                var dt = dtProdotti.Select("FasciaScadenza='" + fasciaScadenza + "'").OrderByDescending(r => r.Field<decimal>("Controvalore")).CopyToDataTable();

                #region riga con background grigio di intestazione per ogni area

                DataRow rowProdottiPerScadenza = dtProdottiPerScadenza.NewRow();

                rowProdottiPerScadenza["CodiceHexScadenza"] = rowScadenza["CodiceHexScadenza"];

                rowProdottiPerScadenza["Prodotto"] = rowScadenza["FasciaScadenza"];
                rowProdottiPerScadenza["Controvalore"] = dt.AsEnumerable().Sum(x => x.Field<decimal>("Controvalore"));
                rowProdottiPerScadenza["Peso"] = dt.AsEnumerable().Sum(x => x.Field<decimal>("Peso"));

                rowProdottiPerScadenza["FontBold"] = true;
                rowProdottiPerScadenza["ReplaceIfZero"] = stringReplaceIfZero;
                rowProdottiPerScadenza["CanLastRow"] = false;
                rowProdottiPerScadenza["IsTotal"] = true;

                dtProdottiPerScadenza.Rows.Add(rowProdottiPerScadenza);

                #endregion

                // I prodotti della dtProdotti vengono mergiati sulla dtProdottiPerArea
                dtProdottiPerScadenza.Merge(dt);

            }

            var dtProdottiPerScadenzaFooter = new DataTable();

            #region Footer
            dtProdottiPerScadenzaFooter.Columns.Add(new DataColumn("Prodotto", typeof(string)));
            dtProdottiPerScadenzaFooter.Columns.Add(new DataColumn("Controvalore", typeof(string)));
            dtProdottiPerScadenzaFooter.Columns.Add(new DataColumn("Peso", typeof(string)));

            dtProdottiPerScadenzaFooter.Rows.Add(       "Totale titoli obbligazionari",
                                                        Helper.FormatCurrency(dataScadenze.distribuzione.totaleRapprNoCC.ToString()),
                                                        (dataScadenze.distribuzione.pesoTot * 100).ToString()
                                                    );
            #endregion

            #endregion

            #endregion

            ds.Tables.Add(dtPatrimonio);
            if (dtScadenze.Rows.Count > 0)
                ds.Tables.Add(dtScadenze.AsEnumerable().OrderBy(r => r.Field<int>("Ordinamento")).CopyToDataTable());
            else
                ds.Tables.Add(dtScadenze);
            ds.Tables.Add(dtProdotti);
            ds.Tables.Add(dtProdottiPerScadenza);
            ds.Tables.Add(dtProdottiPerScadenzaFooter);

            return ds;
        }

     
        public virtual bool isControvalore(){return !GetOption<Opzione3>().Valore;}
        public virtual bool presenzaGraficoBarre(){return GetOption<Opzione4_1>().Valore;}
        public virtual bool presenzaTabella() { return GetOption<Opzione4_2>().Valore; }

        public virtual string getTesto1()
        {
            return !isControvalore() ?
                Helper.ReplaceVariables(GetText().Rows[0]["testo1"].ToString(), base.EnvironmentFacade.ReportEnvironment) :
                Helper.ReplaceVariables(GetText().Rows[0]["testo2"].ToString(), base.EnvironmentFacade.ReportEnvironment);
        }
        public virtual string getNote1()
        {
            string nota = GetText().Rows[0]["testo4"].ToString();


            var dataDistribuzioneScadenze = datiSeiUnico.patrimonioUnit().patrimonioCasa.distribuzioneScadenze;

            decimal GC = dataDistribuzioneScadenze.distribuzione.coverage * 100; // grado di copertura della distribuzione

            var listaValori = new List<decimal>() 
            { 
               (GC != 0 && GC != 100) ? GC : 0
            };

            var listaValoriStringa = new List<string>() 
            { 
                Helper.FormatPercentage(GC, 2)               
            };

            return BuildDynamicNote(nota, listaValori, listaValoriStringa);
        }
        public virtual string getTesto2()
        {
           return Helper.ReplaceVariables(GetText().Rows[0]["testo3"].ToString(), EnvironmentFacade.ReportEnvironment);
        }
        
    }
}
